package com.link.core.basic.service.snowflake;

import com.alibaba.fastjson.JSONObject;
import com.link.core.basic.init.IdInit;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.concurrent.ThreadLocalRandom;
import java.util.concurrent.atomic.AtomicLong;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

public class IdWorkerSingleton {

   private static final Logger logger = LogManager.getLogger(IdWorkerSingleton.class);
   private static volatile IdWorkerSingleton instance;
   private static final long START_TIMESTAMP = 1541001600000L;
   private static final long MACHINE_BIT = 5L;
   private static final long DATACENTER_BIT = 5L;
   private static final long SEQUENCE_BIT = 12L;
   private static final long MAX_DATACENTER_NUM = 31L;
   private static final long MAX_MACHINE_NUM = 31L;
   private static final long MAX_SEQUENCE = 4095L;
   private static final long MACHINE_LEFT = 12L;
   private static final long DATA_CENTER_LEFT = 17L;
   private static final long TIMESTAMP_LEFT = 22L;
   private long dataCenterId;
   private long machineId;
   private long sequence = 0L;
   private static AtomicLong atomicSequence = new AtomicLong(0L);
   private long lastTimeStamp = -1L;


   public static IdWorkerSingleton getInstance(Long dataCenterId, long machineId) {
      if(null == instance) {
         Class var3 = IdWorkerSingleton.class;
         synchronized(IdWorkerSingleton.class) {
            if(null == instance) {
               if(!IdInit.idInitExecFlag()) {
                  dataCenterId = Long.valueOf((long)ThreadLocalRandom.current().nextInt(32));
                  machineId = (long)ThreadLocalRandom.current().nextInt(32);
                  logger.error("还未初始化好idnum，先走随机数，当前生成的随机数为：" + machineId + "," + dataCenterId);
                  return new IdWorkerSingleton(dataCenterId.longValue(), machineId);
               }

               logger.info("设置当前机器的机器ID和数据中心ID：" + machineId + "," + dataCenterId);
               instance = new IdWorkerSingleton(dataCenterId.longValue(), machineId);
            }
         }
      }

      return instance;
   }

   private IdWorkerSingleton(long dataCenterId, long machineId) {
      if(dataCenterId <= 31L && dataCenterId >= 0L) {
         if(machineId <= 31L && machineId >= 0L) {
            this.dataCenterId = dataCenterId;
            this.machineId = machineId;
         } else {
            throw new IllegalArgumentException("机器标识超出最大值或小于0");
         }
      } else {
         throw new IllegalArgumentException("数据中心ID超出最大值或小于0");
      }
   }

   public synchronized long nextId() {
      long currentTimeStamp = this.getNewStamp();
      if(currentTimeStamp < this.lastTimeStamp) {
         throw new RuntimeException("时钟后移。停止生成ID");
      } else {
         if(currentTimeStamp == this.lastTimeStamp) {
            atomicSequence.getAndIncrement();
            if(atomicSequence.compareAndSet(4096L, 0L)) {
               currentTimeStamp = this.getNextMillis(this.lastTimeStamp);
            }
         } else {
            atomicSequence.getAndIncrement();
            atomicSequence.compareAndSet(4096L, 0L);
         }

         this.lastTimeStamp = currentTimeStamp;
         return currentTimeStamp - 1541001600000L << 22 | this.dataCenterId << 17 | this.machineId << 12 | atomicSequence.get();
      }
   }

   private long getNextMillis(long lastTimeStamp) {
      long millis;
      for(millis = this.getNewStamp(); millis <= lastTimeStamp; millis = this.getNewStamp()) {
         ;
      }

      return millis;
   }

   private long getNewStamp() {
      return System.currentTimeMillis();
   }

   public static void main(String[] args) {
      IdWorkerSingleton id = getInstance(Long.valueOf(0L), 31L);
      Long l = Long.valueOf(id.nextId());
      System.out.println(l);
      System.out.println("----------------------------------------------");
      System.out.println(getDatacenterId(l));
      System.out.println(getMachineId(l));
      System.out.println(getSequence(l));
      System.out.println("----------------------------------------------");
      System.out.println(parseInfo(l.toString()).toJSONString());
   }

   public static String dateToStamp(String s) {
      SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
      Date date = null;

      try {
         date = simpleDateFormat.parse(s);
      } catch (ParseException var6) {
         logger.error(var6);
      }

      long ts = 0L;
      if(null != date) {
         date.getTime();
      }

      String res = String.valueOf(ts);
      return res;
   }

   public static long getMachineId2(Long id) {
      Long machineId = Long.valueOf(id.longValue() >> 12 & 31L);
      return machineId.longValue();
   }

   public static long getDatacenterId2(Long id) {
      Long dataCenterId = Long.valueOf(id.longValue() >> 17 & 31L);
      return dataCenterId.longValue();
   }

   public static long getMachineId(Long id) {
      long workIdShift = id.longValue() >>> 12;
      long workId = workIdShift & getBitMask(Long.valueOf(5L));
      return workId;
   }

   public static long getDatacenterId(Long id) {
      long dataCenterIdShift = id.longValue() >>> 17;
      long dataCenterId = dataCenterIdShift & getBitMask(Long.valueOf(5L));
      return dataCenterId;
   }

   public static long getSequence(Long id) {
      long sequence = id.longValue() & getBitMask(Long.valueOf(12L));
      return sequence;
   }

   public static long getTimeStamp(Long id) {
      long timeStampResult = id.longValue() >>> 22;
      long timeStamp = 1541001600000L + timeStampResult;
      return timeStamp;
   }

   private static long getBitMask(Long bit) {
      return ~(-1L << (int)bit.longValue());
   }

   public static JSONObject parseInfo(String id) {
      JSONObject jsonObject = new JSONObject();
      jsonObject.put("id", id);
      id = Long.toBinaryString(Long.parseLong(id));
      int len = id.length();
      jsonObject.put("binary", id);
      int sequenceStart = (int)((long)len < 12L?0L:(long)len - 12L);
      int workerStart = (int)((long)len < 17L?0L:(long)len - 17L);
      int timeStart = (int)((long)len < 22L?0L:(long)len - 22L);
      String sequence = id.substring(sequenceStart, len);
      String machineId = sequenceStart == 0?"0":id.substring(workerStart, sequenceStart);
      String dataCenterId = workerStart == 0?"0":id.substring(timeStart, workerStart);
      String time = timeStart == 0?"0":id.substring(0, timeStart);
      int sequenceInt = Integer.valueOf(sequence, 2).intValue();
      int machineIdInt = Integer.valueOf(machineId, 2).intValue();
      int dataCenterIdInt = Integer.valueOf(dataCenterId, 2).intValue();
      jsonObject.put("dataCenterId", Integer.valueOf(dataCenterIdInt));
      jsonObject.put("machineId", Integer.valueOf(machineIdInt));
      jsonObject.put("sequence", Integer.valueOf(sequenceInt));
      long diffTime = Long.parseLong(time, 2);
      Long timeLong = Long.valueOf(diffTime + 1541001600000L);
      jsonObject.put("time", stampToDate(timeLong.toString()));
      jsonObject.put("timestamp", timeLong);
      return jsonObject;
   }

   private static String stampToDate(String s) {
      SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
      long lt = (new Long(s)).longValue();
      Date date = new Date(lt);
      String res = simpleDateFormat.format(date);
      return res;
   }

}
